{
    This file is part of Tedalo.

    Tedalo converts data sent from Keller GSM-2/ARC-1 to TOA5 file format.

    Copyright (C) 2011-2020  Corax Solution AB.

    Tedalo is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Tedalo is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Tedalo.  If not, see <http://www.gnu.org/licenses/>.

    Note that Tedalo depends on other software, with different licensing
    conditions.

    For more information, contact Jrg Schiemann, josc@corax.nu
}

unit fmMain;
// Tedalo = Teroc Data Logger
interface

uses
  Windows, Forms, Classes, StdCtrls, Controls, ExtCtrls,
  klSettings;

type
  TfrmMain = class(TForm)
    mmoInfo: TMemo;
    tmrMain: TTimer;
    pnlStatus: TPanel;
    lblCounter: TLabel;
    pnlTop: TPanel;
    btnRun: TButton;
    btnAbout: TButton;
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnRunClick(Sender: TObject);
    procedure tmrMainTimer(Sender: TObject);
    procedure mmoInfoChange(Sender: TObject);
    procedure btnAboutClick(Sender: TObject);
  private
    { Private declarations }
    LoopCounter: Int64;
    Settings: TSettings;
    procedure GetNewLogData;
  protected
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation
{$R *.dfm}

uses
  Messages, SysUtils, Variants, Graphics, Dialogs, fmAbout,
  dmSystem, klGSM2Mail, klGSM2Data, klTOA5, utDate,
  utSystem, utLog;

{-------------------------------------------------------------------------------
 TfrmMain
-------------------------------------------------------------------------------}
procedure TfrmMain.FormCreate(Sender: TObject);
begin
  Write2Log('  ***** TEDALO START *****');
  LoopCounter := 0;
  Settings := TSettings.getInstance;
  Settings.LoggList := mmoInfo.Lines;
  daSystem := TdaSystem.Create(Self, Settings, mmoInfo.Lines);
  Caption := 'Tedalo ' + GetVersion;
  tmrMain.Enabled := True;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  if Assigned(Settings) then begin
    Settings.Free;
  end;
end;

procedure TfrmMain.GetNewLogData;
var
  GSM2MailBox: T_GSM2MailBox;
  GSM2Mail: T_GSM2Mail;
  GSM2Data: T_GSM2Data;
  TOA5: T_TOA5;
  i: Integer;
  loggFileName: string;
  TimerStatus: Boolean;
  msg: string;
begin
  try
    Screen.Cursor := crHourGlass;
    TimerStatus := tmrMain.Enabled;
    tmrMain.Enabled := False;
    tmrMain.Interval := 1000 * 300;    // 300 seconds = 5 minutes
    LoopCounter := LoopCounter+1;
    Write2Log('Starting Loop ' + IntToStr(LoopCounter));
    lblCounter.Caption := IntToStr(LoopCounter);
    btnRun.Enabled := False;
    Application.ProcessMessages;
    try
      GSM2MailBox := T_GSM2MailBox.Create(Settings);
      try
        mmoInfo.Clear;
        mmoInfo.Lines.Add('Fetching mail ...');
        GSM2MailBox.getMail;
        if GSM2MailBox.MailList.Count > 0 then begin
          daSystem.Connect;
          for i:= 0 to GSM2MailBox.MailList.Count - 1 do begin
            GSM2Mail := T_GSM2Mail(GSM2MailBox.MailList.Objects[i]);
            mmoInfo.Lines.Add('***** MAIL No.: ' + IntToStr(GSM2Mail.IndexNr) + ' *****');
            GSM2Data := T_GSM2Data.Create(Settings);
            try
              if GSM2Data.SaveGSM2DataToDB(GSM2Mail) then begin
                GSM2Mail.IsImportet := True;
              end;
            finally
              GSM2Data.Free;
            end;
          end;
          daSystem.Disconnect;
          if Settings.DoDeleteImportedEMails then begin
            GSM2MailBox.DeleteImportedMailsFromServer;
          end;
        end;
      finally
        GSM2MailBox.Free;
      end;
      // export the measured values from the db to a toa5 file(s)
      TOA5 := T_TOA5.Create(Settings);
      try
        if TOA5.OpenJobs.Count > 0 then begin
          TOA5.exportMeasuringValues;
        end;
      finally
        TOA5.Free;
      end;
    finally
      mmoInfo.Lines.Add('DONE!');
      loggFileName := ChangeFileExt(Application.ExeName, '.log');
      mmoInfo.Lines.SaveToFile(loggFileName);
      tmrMain.Enabled := TimerStatus;
      btnRun.Enabled := True;
      Screen.Cursor := crDefault;
    end;
  except
    on E: Exception do begin
        msg := 'Error at frmMain.GetNewLogData:' + E.Message;
        mmoInfo.Lines.Add(msg);
    end;
  end;
end;

procedure TfrmMain.btnRunClick(Sender: TObject);
begin
  GetNewLogData;
end;

procedure TfrmMain.tmrMainTimer(Sender: TObject);
begin
  GetNewLogData;
end;

procedure TfrmMain.mmoInfoChange(Sender: TObject);
var
  sLine: string;
begin
  if mmoInfo.Lines.Count > 0 then begin
    sLine := mmoInfo.Lines.Strings[mmoInfo.Lines.Count-1];
    Write2Log(sLine);
  end;
end;

procedure TfrmMain.btnAboutClick(Sender: TObject);
begin
  with TfrmAbout.Create(Self) do begin
    try
      ShowModal;
    finally
      Free;
    end;
  end;
end;

end.
