{
    This file is part of Tedalo.

    Tedalo converts data sent from Keller GSM-2/ARC-1 to TOA5 file format.

    Copyright (C) 2011-2020  Corax Solution AB.

    Tedalo is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Tedalo is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Tedalo.  If not, see <http://www.gnu.org/licenses/>.

    Note that Tedalo depends on other software, with different licensing
    conditions.

    For more information, contact Jrg Schiemann, josc@corax.nu
}

unit klGSM2Data;

interface

uses
  Classes,
  klParse, klMemoryPage, klSettings, klJob, klGSM2Mail;

type
  T_GSM2Data = class(TObject)
  private
    fParser : T_Parser;
    fLoggList : TStrings;
    fGSMType: string;
    fSettings: TSettings;
    fJob: T_Job;
    fInsertedValues: Integer;
    fDuplicates: Integer;
    fDiscards: Integer;
    function DecodeBase64(cipherText: string) : string;
    function Decode64ByteArray(cipherText : string): ByteArray;
    function Get_dBm(input: string) : string;
    procedure saveValue(ch: Integer; val:Double; time: TDateTime);
    procedure addToLogg(sMsg: string);
    function getJob: T_Job;
  public
    constructor Create(Settings: TSettings);
    destructor Destroy; override;
    function SaveGSM2DataToDB(GSM2Mail: T_GSM2Mail): Boolean;
    property LoggList: TStrings read fLoggList write fLoggList;
    property GSMType: string read fGSMType;
    property Job: T_Job read getJob;
    property InsertedValues: Integer read fInsertedValues; // how many values has been imported
    property Duplicates: Integer read fDuplicates;         // counter for duplicate values job-channel-timesstamp-value
    property Discard: Integer read fDiscards;
  end;

implementation

uses
  SysUtils, Forms, IniFiles, Windows,
  utString, utLog, dmSystem, IB_Session, IB_Header;

{-------------------------------------------------------------------------------
 T_GSM2Data
-------------------------------------------------------------------------------}
constructor T_GSM2Data.Create(Settings: TSettings);
begin
  inherited Create;
  fDuplicates := 0;
  fInsertedValues := 0;
  fDiscards := 0;
  fSettings := Settings;
  fLoggList := Settings.LoggList;
  fParser := T_Parser.Create;
end;

destructor T_GSM2Data.Destroy;
begin
  if Assigned(fParser) then begin
    fParser.Free;
  end;
  if Assigned(fJob) then begin
    fJob.Free;
  end;
  inherited;
end;

function T_GSM2Data.DecodeBase64(cipherText: string): string;
const
  pad = '=';
  base64Chars ='ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
var
  i, PadCount : integer;
  Block : string[3];
  x1, x2, x3 : byte;

  function Char2IDx( c: char ): byte;
  var a : integer;
  begin
    for a := 1 to Length( base64Chars ) do begin
      if base64Chars[a] = c then begin
        Result := pred( a );
        Exit;
      end;
    end;
    Result := Ord(Pad);
  end;
begin
  // input _must_ be at least 4 chars long,
  // or multiple of 4 chars
  i := Length(cipherText);
  if ( i < 4 ) or ( i mod 4 <> 0 ) then
    Result := ''
    // raise EBase64Error.Create( 'Base64ToStr: illegal input length!' );
  else begin
    PadCount := 0;
    i := Length(cipherText);
    // count padding chars, if any
    while (cipherText[i] = Pad) and (i > 0) do begin
      inc(PadCount);
      dec(i);
    end;
    Result := '';
    i := 1;
    SetLength(Block,3);
    while i <= Length(cipherText) - 3 do begin
      // reverse process of above
      //OutputDebugString(PChar('Durchlauf =' + IntToStr(i)));
      x1 := (Char2Idx(cipherText[i]) shl 2) or (Char2IDx(cipherText[i + 1]) shr 4);
      Result := Result + Chr(x1);
      x2 := (Char2Idx(cipherText[i + 1]) shl 4) or (Char2IDx(cipherText[i + 2]) shr 2);
      Result := Result + Chr(x2);
      x3 := (Char2Idx(cipherText[i + 2]) shl 6) or (Char2IDx(cipherText[i + 3]));
      Result := Result + Chr(x3);
      inc(i,4);
    end;

    // delete padding, if any
    while PadCount > 0 do  begin
      Delete( Result, Length(Result), 1 );
      dec(PadCount);
    end;
  end;
end;

function T_GSM2Data.Decode64ByteArray(cipherText: string): ByteArray;
var
  tmp : String;
  i : Integer;
  arr : ByteArray;
begin
  tmp:=DecodeBase64(cipherText);
  setLength(arr,Length(tmp));
  for i:= 0 to Length(tmp) -1 do
    arr[i] := ord(tmp[i+1]);
  result := arr;
end;

procedure T_GSM2Data.saveValue(ch: Integer; val: Double; time: TDateTime);
var
  hour, minute, second, ms: Word;
  t: TDateTime;
  msg: string;
begin
  DecodeTime(time,hour,minute,second,ms);
  t := EncodeTime(hour,minute,second,0);
  t := t + Trunc(time);
  with daSystem.dsqlMeasures do begin
    if Prepared then
      Unprepare;
    ParamByName('m_job').AsString := Job.Nr;
    ParamByName('m_channel_nr').AsInteger := ch;
    ParamByName('m_time').AsDateTime := t;
    ParamByName('m_value').AsDouble := val;
    try
      Execute;
      fInsertedValues := fInsertedValues + 1;
    except
       on E: EIB_ISCError do begin
         case e.ERRCODE of
           isc_no_dup: begin
             fDuplicates := fDuplicates + 1;
           end;
           else begin
             addToLogg('DB_Error at GSM2Data.saveValue: e.ERRCODE=' + IntToStr(e.ERRCODE) + ' , E.Message=' + E.Message);
           end;
         end; // case
       end;
       on E: Exception do begin
         msg := 'Error at GSM2Data.saveValue: ' + E.Message;
         addToLogg(msg);
       end;
    end;
  end;
end;

procedure T_GSM2Data.addToLogg(sMsg: string);
begin
  if Assigned(LoggList) then begin
    LoggList.Add(sMsg);
    Application.ProcessMessages;
  end;
end;

function T_GSM2Data.getJob: T_Job;
begin
  Result := fJob;
end;

function T_GSM2Data.SaveGSM2DataToDB(GSM2Mail: T_GSM2Mail): Boolean;
var
  commandList : TStringList;
  variables : TStringList;
  values    : TStringList;
  measureValues : TStringList;
  c,v,u,a : Integer;
  recordData : TMemoryData;
  sJobNr: string;
  commandChar: string;
  idx: Integer;
  variableName: string;
  sTmp: string;
  sSignalQualitat: string;
  sBatteryInPro: string;
  sBatteryVoltage: string;
begin
  Result := False;
  sJobNr := getOnlyDigits(GSM2Mail.Subject);
  fJob := T_Job.Create(fSettings, sJobNr);
  if not fJob.Active then begin
    addToLogg('GSM2Data.SaveGSM2DataToDB: Job "' + sJobNr + '" is not aktive! Skipping.');
    Exit;
  end;
  addToLogg('GSM2Data.SaveGSM2DataToDB: decoding Job: ' + sJobNr);
  //addToLogg('MsgUID=' + GSM2Mail.MsgUID);
  //addToLogg('MsgId=' + GSM2Mail.MsgId);
  addToLogg('From:' +GSM2Mail.From);
  addToLogg('Subject: ' + GSM2Mail.Subject);
  //addToLogg(GSM2Mail.Body);

  commandList := fParser.getCommands(fParser.cleanText(GSM2Mail.Body));

  for c := 0 to commandList.Count - 1 do begin
    variables := fParser.getVariables(commandList.Strings[c]);
    try
      if variables.Count = 0 then
        Continue;

      commandChar := variables.Strings[0];

      if ((variables.Strings[0]='C') and (variables.Count>1)) then begin  // /a=1593 /b=30
        //values := fParser.getValues(variables.Strings[1]);
        //values := fParser.getValues(variables.Strings[2]);
      end;

      if ((variables.Strings[0]='I') and (variables.Count=6)) then begin  // #I/n=280/s=16/b=98/f=09.49/v=+3.786 . variables.count=6 (inkl I)
        (*for idx := 0 to variables.Count-1 do begin
          variableName := variables[idx];
        end;
        values := fParser.getValues(variables.Strings[1]);
        if values.Count > 0 then begin
          sTmp := values.Strings[1];
        end;*)
        // Serie number GSM modul = /n
        // values := fParser.getValues(variables.Strings[1]);
        // Signal quality GSM Network [0..31] = /s
        values := fParser.getValues(variables.Strings[2]);
        sSignalQualitat := Get_dBm(values.Strings[1]);
        addToLogg('GSM network signal quality: ' + sSignalQualitat);
        // Batterie in % [0..100] = /b
        values := fParser.getValues(variables.Strings[3]);
        sBatteryInPro := values.Strings[1];
        addToLogg('Battery: ' + sBatteryInPro + ' %');
        // Software version GSM Modul [YY.KW] = /f
        // values := fParser.getValues(variables.Strings[4]);
        // Battery voltage GSM Modul [x.xxx] = /v
        values := fParser.getValues(variables.Strings[5]);
        sBatteryVoltage := values.Strings[1];
        addToLogg('Battery voltage: ' + sBatteryVoltage + ' V');
      end;

      {---------------------------------------------------------------------------
      B    -> BASE64 encoded Record
      ---------------------------------------------------------------------------}
      if ((variables.Strings[0] = 'B') and (variables.Count > 1)) then  begin    // variables.count=70
        values := fParser.getValues(variables.Strings[1]);
        try
          if (values.Strings[0] = 'a') then begin
            measureValues := TMemoryPage.parseRecord(Decode64ByteArray(copy(commandList.Strings[c],5,length(commandList.Strings[c])-4)));
            try
              // delete everything besides  measured data
              for a := (measureValues.Count - 1) downto 0 do begin
                if (TMemoryData(measureValues.Objects[a]).DataType <> 1) then begin
                  recordData := TMemoryData(measureValues.Objects[a]);
                  measureValues.Delete(a);
                  recordData.Free;
                  fDiscards := fDiscards + 1;
                end;
              end;

              u := 0;
              if  (measureValues.Count > 0) then begin
                 while (u < measureValues.Count) do begin
                   recordData := TMemoryData(measureValues.Objects[u]);
                   saveValue(recordData.Channel,recordData.Value,recordData.AbsTime);
                   recordData.Free;
                   u := u + 1;
                 end;
              end;
            finally
              measureValues.Free;
            end;
          end;
        finally
          values.Free;
        end;
      end;
    finally
      variables.Free;
    end
  end;
  addToLogg('Inserted values:  ' + IntToStr(fInsertedValues));
  addToLogg('Duplicate values: ' + IntToStr(fDuplicates));
  //addToLogg('Discarded records: ' + IntToStr(fDiscards));
  if Assigned(commandList) then  begin
    commandList.Free;
  end;
  Result := True;
end;

// The strenght of the signal is declared from 0..31 => -113dbm .. -51 dbm
function T_GSM2Data.Get_dBm(input: string): string;
var
  number, dbm : Integer;
begin
  try
    number := StrToInt(input);
  except
    number := -1;
  end;
  if (number >= 0) then begin
    dbm := -113 + (2 * number);
    Result := IntToStr(dbm) + ' dBm';
  end else begin
    Result := ' dbm';
  end;
end;

end.
