{
    This file is part of Tedalo.

    Tedalo converts data sent from Keller GSM-2/ARC-1 to TOA5 file format.

    Copyright (C) 2011-2020  Corax Solution AB.

    Tedalo is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Tedalo is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Tedalo.  If not, see <http://www.gnu.org/licenses/>.

    Note that Tedalo depends on other software, with different licensing
    conditions.

    For more information, contact Jrg Schiemann, josc@corax.nu
}

unit klGSM2Mail;

interface

uses
  Classes, IdPOP3, IdMessage,
  klSettings;

//#{$DEFINE FAKINGMAIL}

type T_GSM2Mail = class(TObject)
  private
    fMsgUID: string;
    fMsgId: string;
    fFrom: string;
    fSubject: string;
    fBody: string;
    fIndexNr: Integer;
    fInsertedValues: Integer;
    fDuplicates: Integer;
    fIsImported: Boolean;
  public
    constructor Create;
    destructor Destroy; override;
    property MsgUID: string read fMsgUID write fMsgUID;
    property MsgId: string read fMsgId write fMsgId;
    property From: string read fFrom write fFrom;
    property Subject: string read fSubject write fSubject;
    property Body: string read fBody write fBody;
    property IndexNr: Integer read fIndexNr write fIndexNr;
    property InsertedValues: Integer read fInsertedValues write fInsertedValues; // how many values has been imported
    property Duplicates: Integer read fDuplicates write fDuplicates;             // counter for duplicate values of job-channel-timesstamp-value
    property IsImportet: Boolean read fIsImported write fIsImported;
end;

type T_GSM2MailBox = class(TObject)
    IdPop3: TIdPOP3;
    IdMsg: TIdMessage;
  private
    fMailList: TStringList;
    fLoggList: TStrings;
    fSettings: TSettings;
    function getMailCount: Integer;
  protected
  public
    constructor Create(Settings: TSettings);
    destructor Destroy; override;
    function setPOP3Props: Boolean;
    procedure getMail;
    procedure clearMailList;
    procedure addToLogg(sMsg: string);
    procedure DeleteImportedMailsFromServer;
    property MailList: TStringList read fMailList;
    property MailCount: Integer read getMailCount;
    property LoggList: TStrings read fLoggList write fLoggList;
end;

implementation

uses
  IniFiles, SysUtils, Forms,
  IdBaseComponent, IdComponent, IdTCPConnection, IdTCPClient,
  IdExplicitTLSClientServerBase, IdMessageClient,  IdIOHandler,
  IdIOHandlerSocket, IdIOHandlerStack, utTCPIP;

{-------------------------------------------------------------------------------
 T_GSM2Mail
-------------------------------------------------------------------------------}
constructor T_GSM2Mail.Create;
begin
  inherited Create;
  fIsImported := False;
  fIndexNr := 0;
  fDuplicates := 0;
end;

destructor T_GSM2Mail.Destroy;
begin
  inherited;
end;

{-------------------------------------------------------------------------------
 T_GSM2MailBox
-------------------------------------------------------------------------------}

constructor T_GSM2MailBox.Create(Settings: TSettings);
begin
  inherited Create;
  fSettings := Settings;
  fLoggList := Settings.LoggList;
  IdPOP3 := TIdPOP3.Create(nil);
  IdMsg := TIdMessage.Create(nil);
  fMailList:= TStringList.Create;
end;

destructor T_GSM2MailBox.Destroy;
begin
  if Assigned(fMailList) then begin
    clearMailList;
    fMailList.Free;
  end;
  if Assigned(IdPop3) then begin
    IdPop3.Free;
  end;
  if Assigned(IdMsg) then begin
    IdMsg.Free;
  end;
  inherited Destroy;
end;

procedure T_GSM2MailBox.clearMailList;
var
  i: Integer;
begin
  for i := 0 to fMailList.Count - 1 do begin
    fMailList.Objects[i].Free;
  end;
  fMailList.Clear;
end;


function T_GSM2MailBox.setPOP3Props: Boolean;
var
  IPAddress: string;
begin
  Result := False;
  IPAddress := GetIPAddress(fSettings.Mail_POP3_Server);
  if not IsIPAddress(IPAddress) then begin
    addToLogg('Error: (setPOP3Props) - Wrong IPAddress [' + IPAddress + ']');
    Exit;
  end;
  addToLogg('IPAddress=[' + IPAddress + ']');
  IdPop3.Host := IPAddress;
  IdPop3.Port := 110;
  //IdPop3.AutoLogin := False;
  // maximum time to wait for the conenction to the SMTP server
  IdPop3.ConnectTimeout := 12000; // time in milleseconds !
  // The POP3 protocol defines a greeting command that the server is required to send to the client upon initial connection
  // for successful completion of methods that perform read operations using the client.
  IdPop3.ReadTimeout := 10000;    // time in milleseconds !
  IdPop3.Username := fSettings.Mail_Username;
  IdPop3.Password := fSettings.Mail_Password;
  Result := True;
end;

procedure T_GSM2MailBox.addToLogg(sMsg: string);
begin
  if Assigned(LoggList) then begin
    LoggList.Add(sMsg);
    Application.ProcessMessages;
  end;
end;

procedure T_GSM2MailBox.getMail;
var
  GSM2Mail: T_GSM2Mail;
  MsgUID: TStringList;
  MsgCount, MailIdx: Integer;
begin
  MsgUID := TStringList.Create;
  {$IFDEF FAKINGMAIL}
    try
      MsgUID.LoadFromFile('GsmMails\gsmmail001.txt');
      addToLogg('Faking Mail.');
      GSM2Mail := T_GSM2Mail.Create;
      GSM2Mail.IndexNr := 1;
      GSM2Mail.MsgUID  := '1';
      GSM2Mail.MsgId   := '1';
      GSM2Mail.Subject := '+46727410082 M';
      GSM2Mail.From    := 'gsm@teroc.se';
      GSM2Mail.Body    := MsgUID.Text;
      fMailList.AddObject(IntToStr(MailIdx), GSM2Mail);
    finally
      MsgUID.Free;
    end
  {$ELSE}
    try
      try
        if not SetPOP3Props then begin
          Exit;
        end;
        addToLogg('Connecting to ' + IdPop3.Host + ' ...');
        IdPop3.Connect;
        //IdPop3.Login;  // in case: IdPop3.AutoLogin := False
        addToLogg('Connected.');
        MsgCount := IdPOP3.CheckMessages;
        addToLogg('Number of mails on server:' + IntToStr(MsgCount));
        if MsgCount > 0 then begin
          for MailIdx := 1 to MsgCount do begin
            addToLogg('Fetching mail no. ' + IntToStr(MailIdx));
            IdPop3.Retrieve(MailIdx, IdMsg);
            addToLogg('Received');
            IdPop3.UIDL(MsgUID, MailIdx);       //  Each mail should be clearly identifiable via this UIDL, which the POP3 recipient server gives the mail (https://www.ietf.org/rfc/rfc1939.txt)
            GSM2Mail := T_GSM2Mail.Create;
            GSM2Mail.IndexNr := MailIdx;
            GSM2Mail.MsgUID  := MsgUID.Text;
            GSM2Mail.MsgId   := IdMsg.MsgId;
            GSM2Mail.Subject := IdMsg.Subject;
            GSM2Mail.From    := IdMsg.From.Text;
            GSM2Mail.Body    := IdMsg.Body.Text;
            fMailList.AddObject(IntToStr(MailIdx), GSM2Mail);
          end;
        end;
        IdPOP3.Disconnect;
      except
        on E: Exception do begin
          addToLogg('Error at GSM2MailBox.getMail:' + E.Message);
        end;
      end;
    finally
      if Assigned(MsgUID) then begin
        MsgUID.Free;
      end;
    end;
  {$ENDIF}
end;

// Deleting mails which are imported ( with or without relevant values) belonging to active jobs
procedure T_GSM2MailBox.DeleteImportedMailsFromServer;
var
  idx: Integer;
  GSM2Mail: T_GSM2Mail;
begin
  try
    IdPop3.Connect;
    //IdPop3.Login;  // IdPop3.AutoLogin := False
    for idx:=0 to fMailList.Count -1 do begin
      GSM2Mail := T_GSM2Mail(fMailList.Objects[idx]);
      if GSM2Mail.IsImportet then begin
        addToLogg('Deleting mail ' + IntToStr(GSM2Mail.IndexNr));
        IdPop3.Delete(GSM2Mail.IndexNr);
      end;
    end;
    IdPOP3.Disconnect;
  except
    on E: Exception do begin
      addToLogg('Error at GSM2MailBox.deleteImportedMailsFromServer:' + E.Message);
    end;
  end;
end;


function T_GSM2MailBox.getMailCount: Integer;
begin
  Result := 0;
  try
    IdPop3.Connect;
    Result := IdPOP3.CheckMessages;
    IdPOP3.Disconnect;
  except
    on E: Exception do begin
      addToLogg('Error at GSM2MailBox.getMailCount:' + E.Message);
    end;
  end;
end;


end.
