{
    This file is part of Tedalo.

    Tedalo converts data sent from Keller GSM-2/ARC-1 to TOA5 file format.

    Copyright (C) 2011-2020  Corax Solution AB.

    Tedalo is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Tedalo is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Tedalo.  If not, see <http://www.gnu.org/licenses/>.

    Note that Tedalo depends on other software, with different licensing
    conditions.

    For more information, contact Jrg Schiemann, josc@corax.nu
}

unit klMemoryPage;

interface

uses Classes;

type
  ByteArray = Array of Byte;

  TMemoryData = class(TObject)
  private
    fDataType : integer;
    fTimeDiv : integer;
    fChannel : integer;
    fValue : Double;
    fText : WideString;
    fAbsTime : TDateTime;
  public
    constructor Create();
    function  getLowNibble(b: Byte) : integer;
    function  getHighNibble(b: Byte) : integer;
    class function  GsmTimeToDateTime(seconds: int64) : TDateTime;
    function KFloat_RoundTo(value : double; digits : integer) : double;
    function getSingle(bytes : array of byte): Single;
    function addData(data : array of Byte; calcTime : TDateTime) : TDateTime;
    function toString() : WideString;
    property DataType : integer read fDataType;
    property TimeDiv : integer read fTimeDiv;
    property Value : Double read fValue;
    property Text : Widestring read fText;
    property AbsTime : TDateTime read fAbsTime;
    property Channel : integer read fChannel;
  end;

  TMemoryPage = class(TObject)
  private
    class function getData(page: array of Byte) : TStringList;
    class function subArray(arr : array of Byte; low, cnt : integer) : ByteArray;
  public
    class function parseRecord(GsmRecord: array of Byte) : TStringList;
  end;

implementation

uses SysUtils, Math;

{-------------------------------------------------------------------------------
 TMemoryData
-------------------------------------------------------------------------------}
function  TMemoryData.getHighNibble(b: Byte) : integer;
begin
  result := b shr 4;
end;


function  TMemoryData.getLowNibble(b: Byte) : integer;
begin
  result := b and $0F;
end;

class function TMemoryData.GsmTimeToDateTime(seconds: int64) : TDateTime;
var
  ms: Extended;
begin
  ms := seconds;
  ms := ms * 1000;
  ms := ms + 63082368000000; // the GSM-Modul is running with UTC-Time

  result := TimeStampToDateTime(MSecsToTimeStamp(ms));
end;


function TMemoryData.KFloat_RoundTo(value : double; digits : integer) : double;
var
  m : int64;
begin
  m := Trunc(value);
  if (value = 0) then begin
    result := 0
  end else if (m = 0) then begin
    result := Math.RoundTo(value, -1*digits)
  end else begin
    m := Trunc(Math.Log10(abs(m))) + 1;
    result := Math.RoundTo(value, -1*digits + m);
  end;
end;

function TMemoryData.getSingle(bytes : array of byte) : Single;
var
R : record case byte of
  1 : (X : single) ;
  2 : (B1,B2,B3,B4 : byte) end ;
begin
  r.B1 := bytes[3];
  r.B2 := bytes[2];
  r.B3 := bytes[1];
  r.B4 := bytes[0];

  result:= r.X;
end;


constructor TMemoryData.Create();
begin
  inherited create();
  fDataType := 0;
  fTimeDiv := 0;
  fValue := 0;
  fText := '';
  fAbsTime := 0;
end;


function TMemoryData.addData(data : array of Byte; calcTime : TDateTime) : TDateTime;
var
  higher, lower : integer;
  measureValue : array[0..3] of Byte;
begin
  result := calcTime;

  if (length(data)=4) then begin
    higher := getHighNibble(data[0]);
    lower :=  getLowNibble(data[0]);

    case (higher) of
      0..14 : begin
                fDataType := 1;
                fChannel := higher;
                measureValue[0] := data[1];
                measureValue[1] := data[2];
                measureValue[2] := data[3];
                measureValue[3] := $00;

                try
                  fValue := KFloat_RoundTo(getSingle(measureValue),6);
                Except on e : Exception do
                  fValue := 999999999;
                end;

                fAbsTime := calcTime + lower*1/(24*60*60);
                result := fAbsTime;
              end;

      15 :    begin
                case (lower) of
                  0 : begin
                        fDataType := 2;
                        fTimeDiv := 65536*data[1]+256*data[2]+data[3];
                        fAbsTime := calcTime + fTimeDiv*1/(24*60*60);
                        result := fAbsTime;
                      end;
                  4 : begin
                        fDataType := 3;
                        fText := char(data[1])+char(data[2])+char(data[3]);
                      end;
                  15: begin
                        fDataType := 4;   // EOR
                      end;
                end;
              end;
    end;
  end;
end;


function TMemoryData.toString() : WideString;
var
  str : WideString;
begin
  str := '';
  case (fDataType) of
    1 : str := 'Data: ' + FloatToStr(fValue) + ' Time:' + DateTimeToStr(fAbsTime) + ' TimeDelta: ' + IntToStr(fTimeDiv);
    2 : str := 'TimeDelta: ' + IntToStr(fTimeDiv) + ' Time:' + DateTimeToStr(fAbsTime);
    3 : str := 'Text: ' + fText;
    4 : str := 'EOP/EOR';
  end;
  result := str;
end;


{-------------------------------------------------------------------------------
 TMemoryPage
-------------------------------------------------------------------------------}
class function TMemoryPage.parseRecord(GsmRecord: array of Byte) : TStringList;
var
  data, page : TStringList;
  i : integer;
  bt : ByteArray;
begin
  data := TStringList.Create();

  i := 0;
  while (length(GsmRecord)>i) do begin
    bt := subArray(GsmRecord, i, 64);
    page := getData(bt);
    data.AddStrings(page);
    page.Clear();
    page.Free();
    i := i + 64;
  end;
  result := data;
end;


class function TMemoryPage.subArray(arr: array of Byte; low: Integer; cnt: Integer) : ByteArray;
var
  a : ByteArray;
  i : integer;
begin
  i := 0;
  while (i<cnt) and (i<length(arr)) do begin
    setLength(a,i+1);
    a[i] := arr[low+i];
    i := i + 1;
  end;
  result := a;
end;


class function TMemoryPage.getData(page: array of Byte) : TStringList;
var
  md : TMemoryData;
  i : integer;
  eop : boolean;
  t : TDateTime;
  data : array[0..3] of Byte;
  fData : TStringList;
  pageLength : integer;
begin
  fData := TStringList.Create();
  eop := false;

  t := TMemoryData.GsmTimeToDateTime(16777216*page[2] + 65536*page[3] + 256*page[4] + page[5]);

  i := 8;
  pageLength := length(page);
  while ((not eop) and (pageLength>i+3)) do begin
    data[0] := page[i];
    data[1] := page[i+1];
    data[2] := page[i+2];
    data[3] := page[i+3];
    md := TMemoryData.Create();
    t := md.addData(data,t);

    if (md.fDataType=4) then
      eop := true;

    fData.AddObject(IntToStr(i), md);
    i := i + 4;
  end;
  result := fData;
end;

end.
