{
    This file is part of Tedalo.

    Tedalo converts data sent from Keller GSM-2/ARC-1 to TOA5 file format.

    Copyright (C) 2011-2020  Corax Solution AB.

    Tedalo is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Tedalo is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Tedalo.  If not, see <http://www.gnu.org/licenses/>.

    Note that Tedalo depends on other software, with different licensing
    conditions.

    For more information, contact Jrg Schiemann, josc@corax.nu
}

unit klSettings;

interface

uses
  IniFiles, Classes;

type
  TSettings = class(TObject)
  private
    function getDB_Path: string;
    function getDB_Server: string;
    function getMail_POP3_Server: string;
    function getMail_Username: string;
    function getMail_Password: string;
    function GetOptionDeleteImportedEMails: Boolean;
  protected
    INIFile: TINIFile;
    fJobsList: TStringList;
    fLoggList: TStrings;
    constructor Create;
  public
    class function getInstance(): TSettings;
    destructor Destroy; override;
    function getJobsList: TStringList;
    function JobIsActive(JobNr: string): Boolean;
    function getJobGSMType(JobNr: string): string;
    function getJobGSMChannelList(GSMType: string): TStringList;
    function getJobTOA5FileName(JobNr: string): string;
    function getVDVSiteName(JobNr: string): string;
    property JobsList: TStringList read getJobsList;
    property LoggList: TStrings read fLoggList write fLoggList;
    property DB_Server: string read getDB_Server;
    property DB_Path: string read getDB_Path;
    property Mail_POP3_Server: string read getMail_POP3_Server;
    property Mail_Username: string read getMail_Username;
    property Mail_Password: string read getMail_Password;
    property DoDeleteImportedEMails: Boolean read GetOptionDeleteImportedEMails;
end;

implementation

uses
  SysUtils, Forms,
  utLog;


var g_Settings: TSettings;

{-------------------------------------------------------------------------------
 TSettings
-------------------------------------------------------------------------------}
constructor TSettings.Create;
begin
  inherited;
  INIFile := TINIFile.Create(ChangeFileExt(Application.ExeName, '.ini'));
  fJobsList := TStringList.Create;
  fJobsList.Sorted := True;
  fJobsList.Duplicates := dupIgnore;
end;

class function TSettings.getInstance: TSettings;
begin
  if g_Settings = nil then begin
    g_Settings := TSettings.Create;
  end;
  Result := g_Settings;
end;

destructor TSettings.Destroy;
begin
  if Assigned(INIFile) then begin
    INIFile.Free;
  end;
  if Assigned(fJobsList) then begin
    fJobsList.Free;
  end;
  inherited;
end;

function TSettings.getDB_Server: string;
begin
  Result := INIFile.ReadString('DB', 'Server' , '');
end;

function TSettings.getDB_Path: string;
begin
  Result := INIFile.ReadString('DB', 'Path' , '');
end;

function TSettings.getMail_POP3_Server: string;
begin
  Result := INIFile.ReadString('Mail', 'POP3 Server' , '');
end;

function TSettings.getMail_Username: string;
begin
  Result := INIFile.ReadString('Mail', 'Username' , '');
end;

function TSettings.getMail_Password: string;
begin
  Result := INIFile.ReadString('Mail', 'Password' , '');
end;

function TSettings.GetOptionDeleteImportedEMails: Boolean;
begin
  Result := False;
  if UpperCase(INIFile.ReadString('Mail', 'DeleteImportedEMail' , 'No'))= 'YES' then begin
    Result := True;
  end;
end;

function TSettings.getJobsList: TStringList;
var i: Integer;
    sJob: string;
begin
  if not INIFile.SectionExists('Jobs') then begin
    Write2Log('Settings-Error: Section "Jobs" doesn''t exist in ini-file!');
  end;
  INIFile.ReadSection('Jobs', fJobsList);
  for i := fJobsList.Count-1 downto 0  do begin
    sJob := fJobsList.Strings[i];
    if not JobIsActive(sJob) then begin
      fJobsList.Delete(i);
    end;
  end;
  Result := fJobsList;
end;

function TSettings.JobIsActive(JobNr: string): Boolean;
begin
  Result := False;
  if UpperCase(INIFile.ReadString(JobNr, 'Active' , 'No'))= 'YES' then begin
    Result := True;
  end;
end;

function TSettings.getJobGSMType(JobNr: string): string;
begin
  Result := INIFile.ReadString(JobNr, 'Type', '');
end;

function TSettings.getJobGSMChannelList(GSMType: string): TStringList;
// index=channelno.  name=channelname  value=unit
var
  ChannelList: TStringList;
  ChCounts, ChIdx, p: Integer;
  Channel, ChannelString, ChannelName, ChannelUnit, s: string;
begin
  ChannelList := TStringList.Create;
  ChCounts := INIFile.ReadInteger(GSMType, 'Channels', 10);
  for ChIdx := 0 to ChCounts - 1 do begin
    Channel := 'CH' + IntToStr(ChIdx);
    ChannelString := INIFile.ReadString(GSMType, Channel, '');
    p := Pos('|', ChannelString);
    ChannelName := Copy(ChannelString,1,p-1);
    ChannelUnit := Copy(ChannelString,p+1,Length(ChannelString)-p);
    s := ChannelName + '=' + ChannelUnit;
    ChannelList.Add(s)
  end;
  Result := ChannelList;
end;

function TSettings.getJobTOA5FileName(JobNr: string): string;
begin
  Result := INIFile.ReadString(JobNr, 'Toa5FileName', '');
end;

function TSettings.getVDVSiteName(JobNr: string): string;
begin
  Result := INIFile.ReadString(JobNr, 'VDVSiteName', '');
end;

end.
