{
    This file is part of Tedalo.

    Tedalo converts data sent from Keller GSM-2/ARC-1 to TOA5 file format.

    Copyright (C) 2011-2020  Corax Solution AB.

    Tedalo is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Tedalo is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Tedalo.  If not, see <http://www.gnu.org/licenses/>.

    Note that Tedalo depends on other software, with different licensing
    conditions.

    For more information, contact Jrg Schiemann, josc@corax.nu
}

unit klTOA5;

interface

uses
  Classes, klSettings, klMatrix, klJob;

type
  T_TOA5File = class(TObject)
  private
    fSettings: TSettings;
    fMatrix: T_Matrix;
    fExportApproved: Boolean;
    fAppendFile: Boolean;
    fToa5FileDoesntExist: Boolean;
    fJob: T_Job;
    fFileName: string;
    fFileDate: TDateTime;
    fChannelNameList: TStringList;
    fChannelValueList: TStringList;
    fRowList: TStringList;
    fTOA5Header1: string;
    fTOA5Header2: string;
    fTOA5Header3: string;
    fTOA5Header4: string;
    fLoggList: TStrings;
    fCount: Integer;
    procedure Intialize;
    procedure resetChannelValueList;
    procedure addToLogg(sMsg: string);
  public
    constructor Create(Settings: TSettings; JobNr: string);
    destructor Destroy; override;
    function write2File(): Boolean;
    procedure setExported2True;
    property ExportApproved: Boolean read fExportApproved;
    property AppendFile: Boolean read fAppendFile;
    property Job: T_Job read fJob;
    property FileName: string read fFileName write fFileName;
    property FileDate: TDateTime read fFileDate;
    property ChannelNameList: TStringList read fChannelNameList write fChannelNameList;
    property ChannelValueList: TStringList read fChannelValueList write fChannelValueList;
    property RowList: TStringList read fRowList write fRowList;
    property TOA5Header1: string read fTOA5Header1;   // Environmentline
    property TOA5Header2: string read fTOA5Header2;   // Field names
    property TOA5Header3: string read fTOA5Header3;   // Units
    property TOA5Header4: string read fTOA5Header4;   // data process
    property LoggList: TStrings read fLoggList write fLoggList;
    property Count: Integer read fCount;              // antal mtvrdar att exportera
  end;

  T_TOA5 = class(TObject)
  private
    fSettings: TSettings;
    fLoggList: TStrings;
    fOpenJobs: TStringList;
    procedure getOpenJobs;
    procedure addToLogg(sMsg: string);
  public
    constructor Create(Settings: TSettings);
    destructor Destroy; override;
    procedure exportMeasuringValues;
    property OpenJobs: TStringList read fOpenJobs;
    property LoggList: TStrings read fLoggList write fLoggList;
  end;

implementation

uses
  SysUtils,
  IniFiles,
  Forms,
  DateUtils,
  IB_Components,
  dmSystem,
  klVDV,
  utDate,
  utFileOps,
  utLog;

{-------------------------------------------------------------------------------
 T_TOA5File
-------------------------------------------------------------------------------}

constructor T_TOA5File.Create(Settings: TSettings; JobNr: string);   // (GSMType: string)
begin
  inherited Create;
  fSettings := Settings;
  fLoggList := Settings.LoggList;
  fMatrix := T_Matrix.Create;
  fJob := T_Job.Create(Settings, JobNr);
  fRowList := TStringList.Create;
  fChannelNameList := TStringList.Create;
  fChannelValueList:= TStringList.Create;
  fExportApproved := False;
  fAppendFile := False;
  fToa5FileDoesntExist := False;
  Intialize;
end;

procedure T_TOA5File.Intialize;
var
  idx: Integer;
  ChannelName, ChannelUnit: string;
  DateCreated, DateLastAccessed, TOA5DateLastModified, VdvLatestJobImport: TDateTime;
  VdV: T_VDV;
  s: string;
begin
  fTOA5Header1 := '"TOA5","Station","Tedalo","","","","","Table" ';
  fTOA5Header2 := '"TIMESTAMP","RECORD"';
  fTOA5Header3 := '"TS","RN"';
  fTOA5Header4 := '"",""';

  // Are there measure values for this job to export?  Should be, because of OpenJobs
  with TIB_Cursor.Create(nil) do begin
    try
      IB_Connection := daSystem.cnTedalo;
      IB_Transaction:= daSystem.tnMain;
      SQL.Text := 'SELECT count(*) ' +
                  'FROM measures ' +
                  'WHERE exp2toa5 = 0 ' +
                  'AND m_job = ' + QuotedStr(fJob.Nr);
      Execute;
      First;
      fCount := Fields[0].AsInteger;
    finally
      Free;
    end;
  end;
  if fCount = 0 then begin
    Exit;
  end;

  fFileName := fSettings.getJobTOA5FileName(fJob.Nr);
  if FileExists(fFileName) and (Length(fJob.VDVSiteName) = 0) then begin
    // without a VDVSiteName the program appends the data to the file without checking the VDV's database
    fAppendFile := True;
    fExportApproved := True;
  end else if not FileExists(fFileName) then begin
    // if no toa5 file exist, it has to be created.
    fExportApproved := True;
    fToa5FileDoesntExist := True;
  end else if GetFileTimes(fFileName, DateCreated, DateLastAccessed, TOA5DateLastModified) then begin
    VdV:= T_VDV.Create(fSettings, fJob.Nr);
    if ( (not fToa5FileDoesntExist) and (not VdV.IsRunning)) then begin
      addToLogg('Cancel export to TOA5');
      Exit;
    end;
    try
      VdvLatestJobImport := VdV.LatestJobImport;
      if VdvLatestJobImport > TOA5DateLastModified then begin   // 13.10
        // If VDV has not imported the previous toa5 file, or VDV (db.robot.c) is not running,
        // VdV.LatestImport will be 1900-01-01 12:00 (2.5); there is no VDV-log entry.
        // and the last toa5 file shouldn't be overwritten.
        s := DateTimeToStr(Now);
        if IncMinute(Now, VdV.MinutesToWait) > VdvLatestJobImport then begin
          Write2Log('New toa5 creation possible: Latest job vimport av VDV:' + DateTimeToStr(VdvLatestJobImport) + ' > TOA5 File date: ' + DateTimeToStr(TOA5DateLastModified) );
          fExportApproved := True;
        end else begin
          addToLogg('VDV import is in progress. Skipping export this time.');
        end;
      end else begin

        addToLogg('TOA5 file is newer then latest VDV import');
        addToLogg('TOA5 File date   : ' + DateTimeToStr(TOA5DateLastModified));
        addToLogg('Latest VDV import: '  + DateTimeToStr(VdvLatestJobImport));
      end;
    finally
      VdV.Free;
    end;
  end;

  if not fExportApproved then begin
    Exit;
  end;

  for idx := 0 to fJob.ChannelList.Count-1 do begin
    ChannelName := fJob.ChannelList.Names[idx];
    ChannelUnit := fJob.ChannelList.ValueFromIndex[idx];
    fTOA5Header2 := fTOA5Header2 + ',"' + ChannelName + '"';
    fTOA5Header3 := fTOA5Header3 + ',"' + ChannelUnit + '"';
    fTOA5Header4 := fTOA5Header4 + ',""';
    fChannelNameList.Add(ChannelName);
    fChannelValueList.Add('"NAN"');
  end;

  fTOA5Header1 := fTOA5Header1 + sLineBreak;
  fTOA5Header2 := fTOA5Header2 + sLineBreak;
  fTOA5Header3 := fTOA5Header3 + sLineBreak;
  fTOA5Header4 := fTOA5Header4 + sLineBreak;
  // Fill Matrix with values from the db
  with TIB_Cursor.Create(nil) do begin
    try
      IB_Connection := daSystem.cnTedalo;
      IB_Transaction:= daSystem.tnMain;
      SQL.Text := 'SELECT m_id, m_channel_nr, m_value, m_time, exp2toa5 ' +
                  'FROM measures ' +
                  'WHERE exp2toa5 = 0 ' +
                  'AND m_job = ' + QuotedStr(fJob.Nr) + ' ' +
                  'ORDER BY m_time, m_channel_nr';
      Execute;
      First;
      while not Eof do begin
        fMatrix.addMValue(Fields[1].AsInteger, Fields[2].AsDouble, Fields[3].AsDateTime);
        Next;
      end;
    finally
      Free;
    end;
  end;
end;

destructor T_TOA5File.Destroy;
begin
  if Assigned(fJob) then
    fJob.Free;
  if Assigned(fMatrix) then
    fMatrix.Free;
  if Assigned(fChannelNameList) then
    fChannelNameList.Free;
  if Assigned(fChannelValueList) then
    fChannelValueList.Free;
  if Assigned(fRowList) then
    fRowList.Free;
  inherited;
end;

procedure T_TOA5File.resetChannelValueList;
var
  idx: Integer;
begin
  for idx := 0 to fChannelValueList.Count -1 do begin
    fChannelValueList.Strings[idx] := '"NAN"';
  end;
end;

procedure T_TOA5File.addToLogg(sMsg: string);
begin
  if Assigned(LoggList) then begin
    LoggList.Add(sMsg);
    Application.ProcessMessages;
  end;
end;

function T_TOA5File.write2File(): Boolean;
var
  i, j, k: Integer;
  A_Zeile: T_Zeile;
  A_Mess : T_mess;
  sRow, TmpFileNameWithPath, FileNameWithPath: string;
  fs: TFileStream;
begin
  Result := False;
  if fCount = 0 then begin
    Exit;
  end;
  if not fExportApproved then begin
    addToLogg('Can''t export Job to TOA5; waiting for VDV!');
    Exit;
  end;

  addToLogg('TOA5File.write2File: Values to export: ' + IntToStr(fMatrix.TimeList.Count));
  DecimalSeparator := '.';
  try
    for i := 0 to fMatrix.TimeList.Count - 1 do begin
      A_Zeile := T_Zeile(fMatrix.TimeList.Objects[i]);
      sRow := '"' + FormatDateTime('yyyy-mm-dd hh:nn:ss', UTCToTime(A_Zeile.time)) + '",' + IntToStr(i);
      resetChannelValueList;
      for j := 0 to A_Zeile.f_ch_list.Count-1 do begin
        A_Mess  := T_Mess(A_Zeile.f_ch_list.Objects[j]);
        fChannelValueList.Strings[A_Mess._ch] :=  FloatToStrF(A_Mess._val,ffFixed,12,5);
      end;
      for k := 0 to ChannelValueList.Count-1 do begin
        sRow := sRow + ',' + fChannelValueList.Strings[k];
      end;
      // sRow := sRow + fChannelValueList.CommaText;
      fRowList.Add(sRow);
    end;
  except
    on E: Exception do begin
      addToLogg('T_TOA5File.write2File Error: ' + E.Message);
      // ToDo: check if the fChannelValueList has a suitable channel number!
    end;
  end;

  FileNameWithPath := fFileName;
  if Self.AppendFile then begin
    fs := TFileStream.Create(FileNameWithPath, fmOpenWrite);
  end else begin
    TmpFileNameWithPath := ChangeFileExt(FileNameWithPath, '.tmp');

    fs := TFileStream.Create(TmpFileNameWithPath, fmCreate);
    fs.Write(fTOA5Header1[1], Length(fTOA5Header1));
    fs.Write(fTOA5Header2[1], Length(fTOA5Header2));
    fs.Write(fTOA5Header3[1], Length(fTOA5Header3));
    fs.Write(fTOA5Header4[1], Length(fTOA5Header4));
  end;
  try
    fs.seek(0, soFromEnd);
    fRowList.SaveToStream(fs);
  finally
    FreeAndNil(fs);
  end;

  if not Self.AppendFile then begin
    if FileExists(FileNameWithPath) then begin
      try
        DeleteFile(FileNameWithPath);
      except
        on E: Exception do begin
          addToLogg('Error at TOA5File.write2File (DeleteFile):' + E.Message);
        end;
      end;
    end;

    if not RenameFile(TmpFileNameWithPath, FileNameWithPath) then begin
      try
        DeleteFile(TmpFileNameWithPath);
      except
        on E: Exception do begin
          addToLogg('Error at TOA5File.RenameFile (DeleteFile):' + E.Message);
        end;
      end;
      Exit;
    end;
  end;
  Result:= True;
end;

procedure T_TOA5File.setExported2True;
begin
  with TIB_DSQL.Create(nil) do begin
    try
      IB_Connection := daSystem.cnTedalo;
      IB_Transaction:= daSystem.tnMain;
      SQL.Text := 'UPDATE measures SET exp2toa5=1 WHERE exp2toa5=0 AND m_job=' + QuotedStr(fJob.Nr);
      Prepare;
      Execute;
    finally
      Free;
    end;
  end;

end;

{-------------------------------------------------------------------------------
 T_TOA5
-------------------------------------------------------------------------------}
procedure T_TOA5.addToLogg(sMsg: string);
begin
  if Assigned(LoggList) then begin
    LoggList.Add(sMsg);
    Application.ProcessMessages;
  end;
end;

constructor T_TOA5.Create(Settings: TSettings);
begin
  inherited Create;
  fSettings := Settings;
  fLoggList := Settings.LoggList;
  fOpenJobs := TStringList.Create;
  getOpenJobs;
end;

destructor T_TOA5.Destroy;
begin
  if Assigned(fOpenJobs) then begin
    fOpenJobs.Free;
  end;
  inherited Destroy;
end;

procedure T_TOA5.getOpenJobs;
var
  iniJobList : TStringList;
  sJob: string;
begin
  iniJobList := fSettings.JobsList;
  with TIB_Cursor.Create(nil) do begin
    try
      IB_Connection := daSystem.cnTedalo;
      IB_Transaction:= daSystem.tnMain;
      SQL.Text := 'SELECT DISTINCT m_job ' +
                  'FROM measures ' +
                  'WHERE  exp2toa5 = 0';
      Execute;
      First;
      while not Eof do begin
        sJob := Fields[0].AsString;
        if Pos(sJob, iniJobList.Text) > 0 then begin
          fOpenJobs.Add( Fields[0].AsString );
        end;
        Next;
      end;
    finally
      Free;
    end;
  end;
end;

procedure T_TOA5.exportMeasuringValues;
var
  idxJob: Integer;
  Toa5File: T_TOA5File;
begin
  if OpenJobs.Count = 0 then begin
    addToLogg('TOA5.exportMeasuringValues: There are no values in the database to export!');
    Exit;
  end;
  addToLogg('Jobs waiting for export to TOA5: ' + IntToStr(OpenJobs.Count));
  for idxJob := 0 to OpenJobs.Count-1 do begin
    addToLogg('Job [' + OpenJobs.Strings[idxJob] + ']');
    addToLogg('Exporting measuring values to TOA5');
    Toa5File := T_TOA5File.Create(fSettings, OpenJobs.Strings[idxJob]);
    try
      if Toa5File.ExportApproved then begin
          if Toa5File.write2File then begin
            Toa5File.setExported2True;
          end;
      end;
    finally
      Toa5File.Free;
    end;
  end;
end;


end.
