{
    This file is part of Tedalo.

    Tedalo converts data sent from Keller GSM-2/ARC-1 to TOA5 file format.

    Copyright (C) 2011-2020  Corax Solution AB.

    Tedalo is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Tedalo is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Tedalo.  If not, see <http://www.gnu.org/licenses/>.

    Note that Tedalo depends on other software, with different licensing
    conditions.

    For more information, contact Jrg Schiemann, josc@corax.nu
}

unit klVDV;

interface

uses
  Classes, klSettings, klJob;

type
  T_VDV = class(TObject)
  private
    fSettings: TSettings;
    fJob: T_Job;
    fProcessName: string;
    fMinutesToWait: Word;
    fLoggList: TStrings;
    fIsRunning: Boolean;
    function getVDVLogFileName(datum: TDateTime): string;
    function getLatestJobImportMonth(datum: TDateTime): TDateTime;
    function getLatestJobImport: TDateTime;
    procedure addToLogg(sMsg: string);
  public
    constructor Create(Settings: TSettings; JobNr: string);
    destructor Destroy; override;
    property ProcessName: string read fProcessName;
    property MinutesToWait: Word read fMinutesToWait;  // hur mnga minuter skall lggas p tid LatestImport
    property LatestJobImport: TDateTime read getLatestJobImport;
    property LoggList: TStrings read fLoggList write fLoggList;
    property IsRunning: Boolean read fIsRunning write fIsRunning;
  end;

implementation

uses
  Windows, SysUtils, DateUtils, Forms,
  utSystem, utLog;

{-------------------------------------------------------------------------------
 T_VDV
-------------------------------------------------------------------------------}

constructor T_VDV.Create(Settings: TSettings; JobNr: string);
begin
  inherited Create;
  fMinutesToWait := 5;
  fSettings := Settings;
  fLoggList := Settings.LoggList;
  fProcessName := 'db.robot.c.exe';
  if processExists(fProcessName) then begin
    fIsRunning := True;
  end else begin
    fIsRunning := False;
    addToLogg('Warning: VDV doesn''t seems to be running!');
  end;
  fJob := T_Job.Create(fSettings, JobNr);
end;

destructor T_VDV.Destroy;
begin
  if Assigned(fJob) then begin
    fJob.Free;
  end;
  inherited;
end;

procedure T_VDV.addToLogg(sMsg: string);
begin
  if Assigned(LoggList) then begin
    LoggList.Add(sMsg);
    Application.ProcessMessages;
  end;
end;

function T_VDV.getVDVLogFileName(datum: TDateTime): string;
var
  pid: DWORD;
  ProcessFileName: string;
  logYYYY_mm : string;
begin
  Result := '';
  DateTimeToString(logYYYY_mm, 'yyyy_mm', datum);
  if IsRunning then begin
    pid := getProcessId(fProcessName);
    GetProcessFileName(pid, ProcessFileName);
    Result := ExtractFilePath(ProcessFileName) + 'log\log_' + logYYYY_mm  + '.txt' ;   //      \db.robot.c\log\log_YYYY_mm.txt
  end;
end;

function T_VDV.getLatestJobImportMonth(datum: TDateTime): TDateTime;
var
  logFileName: string;
  slFile: TStringList;
  stream: TStream;
  idx: Integer;
  s, sTime: string;
  Year, Month, Day, Hour, Min, Sec, MSec : Word;
begin
  Result := EncodeDateTime (1900, 1, 1, 12, 0, 0, 0);; // falls noch kein Eintrag im VDV Log. Dann die schon existierende toa5 NICHT berschrieben werden!
  logFileName := getVDVLogFileName(datum);
  if not FileExists(logFileName) then begin
    Write2Log('Can''t find VDV loggfile "' + logFileName + '"');
    Exit;
  end;
  slFile := TStringList.Create;
  try
    stream := TFileStream.Create(logFileName, fmOpenRead or fmShareDenyNone);
    try
      slFile.LoadFromStream(stream);
    finally
      stream.Free;
    end;

    for idx := slFile.Count-1 downto 0 do begin
      if Pos(fJob.VDVSiteName, slFile.Strings[idx]) > 0 then begin
        try
          s := slFile.Strings[idx];
          sTime := Copy(slFile.Strings[idx],1, 17);
          Year := StrToInt(Copy(sTime,1,4));
          Month:= StrToInt(Copy(sTime,5,2));
          Day  := StrToInt(Copy(sTime,7,2));
          Hour := StrToInt(Copy(sTime,10,2));
          Min  := StrToInt(Copy(sTime,13,2));
          Sec  := StrToInt(Copy(sTime,16,2));
          MSec := 0;
          Result := EncodeDateTime (Year, Month, Day, Hour, Min, Sec, MSec);
        except
          on E: Exception do begin
            Write2Log('Error T_VDV.getLatestJobImportMonth:' +  E.Message);
          end;
        end;
        Break;
      end;
    end;
  finally
    slFile.Free;
  end;
end;

function T_VDV.getLatestJobImport: TDateTime;
var
  LastJobImpCurMonth, LastJobImpPrevMonth: TDateTime;
begin
  LastJobImpPrevMonth := getLatestJobImportMonth(IncMonth( Date,-1));
  LastJobImpCurMonth  := getLatestJobImportMonth(Date);
  if LastJobImpCurMonth > LastJobImpPrevMonth then begin
    Result := LastJobImpCurMonth
  end else begin
    Result := LastJobImpPrevMonth;
  end;
end;

end.
