{
    This file is part of Tedalo.

    Tedalo converts data sent from Keller GSM-2/ARC-1 to TOA5 file format.

    Copyright (C) 2011-2020  Corax Solution AB.

    Tedalo is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Tedalo is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Tedalo.  If not, see <http://www.gnu.org/licenses/>.

    Note that Tedalo depends on other software, with different licensing
    conditions.

    For more information, contact Jrg Schiemann, josc@corax.nu
}

unit utSystem;

interface

uses
  Windows;

type
  TPIDList = array of DWORD;

function GetVersion(): String;
function GetProcessName(PID: DWORD; var ProcessName: string): DWORD;
function GetProcessFileName(PID: DWORD; var ProcessFileName: string): DWORD;
function GetProcessList(var ProcessList: TPIDList): DWORD;
function getProcessId(const AExeName: string): DWORD;
function processExists(exeFileName: string): Boolean;
function GetWindowHandleByExeName(const AExeName: string): THandle;

implementation

uses
  PSAPI, TlHelp32, SysUtils;

var
  WindowHandle: THandle;

function GetVersion(): String;
var
  VerInfoSize  : DWord;
  VerInfo      : Pointer;
  VerValueSize : DWord;
  VerValue     : PVSFixedFileInfo;
  Dummy        : DWord;
begin
  VerInfoSize := GetFileVersionInfoSize(PChar(ParamStr(0)), Dummy);
  GetMem(VerInfo, VerInfoSize);
  GetFileVersionInfo(PChar(ParamStr(0)), 0, VerInfoSize, VerInfo);
  VerQueryValue(VerInfo, '', Pointer(VerValue), VerValueSize);
  with VerValue^ do begin
    result := IntTostr(dwFileVersionMS shr 16);
    result := result+'.'+ IntTostr(dwFileVersionMS and $FFFF);
    result := result+'.'+ IntTostr(dwFileVersionLS shr 16);
    //result := result+'.'+ IntTostr(dwFileVersionLS and $FFFF);
  end;
  FreeMem(VerInfo, VerInfoSize);
end;

function GetProcessName(PID: DWORD; var ProcessName: string): DWORD;
var
  dwReturn     : DWORD;
  hProc        : Cardinal;
  buffer       : array[0..MAX_PATH - 1] of Char;
begin
  dwReturn := 0;
  Zeromemory(@buffer, sizeof(buffer));
  hProc := OpenProcess(PROCESS_QUERY_INFORMATION or PROCESS_VM_READ, FALSE, PID);
  if hProc <> 0 then begin
    GetModulebaseName(hProc, 0, buffer, sizeof(buffer));
    ProcessName := (string(buffer));
    CloseHandle(hProc);
  end else begin
    dwReturn := GetLastError;
  end;
  result := dwReturn;
end;

function GetProcessFileName(PID: DWORD; var ProcessFileName: string): DWORD;
var
  dwReturn     : DWORD;
  hProc        : Cardinal;
  buffer       : array[0..MAX_PATH - 1] of Char;
  ProcessName  : string;
  hWind        : Cardinal;
begin
  dwReturn := 0;
  Zeromemory(@buffer, sizeof(buffer));
  hProc := OpenProcess(PROCESS_QUERY_INFORMATION or PROCESS_VM_READ, FALSE, PID);
  if hProc <> 0 then begin
    GetModuleFileNameEx(hProc,0, buffer, sizeof(buffer));
    ProcessFileName := (string(buffer));
    CloseHandle(hProc);
  end else begin
    dwReturn := GetLastError;
  end;
  result := dwReturn;
end;

function GetProcessList(var ProcessList: TPIDList): DWORD;

  function GetOSVersionInfo(var Info: TOSVersionInfo): Boolean;
  begin
    FillChar(Info, SizeOf(TOSVersionInfo), 0);
    Info.dwOSVersionInfoSize := SizeOf(TOSVersionInfo);
    Result := GetVersionEx(TOSVersionInfo(Addr(Info)^));
    if (not Result) then begin
      FillChar(Info, SizeOf(TOSVersionInfo), 0);
      Info.dwOSVersionInfoSize := SizeOf(TOSVersionInfo);
      Result := GetVersionEx(TOSVersionInfo(Addr(Info)^));
      if (not Result) then begin
        Info.dwOSVersionInfoSize := 0;
      end;
    end;
  end;

var
  dwReturn     : DWORD;
  OS           : TOSVersionInfo;
  // EnumProcesses
  PidProcesses : PDWORD;
  PidWork      : PDWORD;
  BufferSize   : Cardinal;
  Needed       : DWORD;
  cntProcesses : Cardinal;
  i            : Cardinal;
  // CreateToolhelp32Snapshot
  hProcSnapShot: THandle;
  pe32         : TProcessEntry32;
  j            : Cardinal;

begin
  dwReturn := 0;
  // What OS are we running on?
  if GetOSVersionInfo(OS) then begin
    if (OS.dwPlatformId = VER_PLATFORM_WIN32_NT) and (OS.dwMajorVersion = 4) then begin
    // WinNT and higher
      Needed := 0;
      BufferSize := 1024;
      GetMem(PidProcesses, BufferSize);
      // make sure memory is allocated
      if Assigned(PidProcesses) then begin
        try
          // enumerate the processes
          if EnumProcesses(PidProcesses, BufferSize, Needed) then begin
            dwReturn := 0;
            cntProcesses := Needed div sizeof(DWORD) - 1;
            PidWork := PidProcesses;
            setlength(ProcessList, cntProcesses);
            // walk the processes
            for i := 0 to cntProcesses - 1 do begin
              ProcessList[i] := PidWork^;
              Inc(PidWork);
            end;
          end else begin // EnumProcesses = False
            dwReturn := GetLastError;
          end;
        finally
          // clean up no matter what happend
          FreeMem(PidProcesses, BufferSize);
        end;
      end else begin // GetMem = nil
        dwReturn := GetLastError;
      end;
    end  else begin // Win 9x and higher except WinNT
      // make the snapshot
      hProcSnapShot := CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
      if hProcSnapShot <> INVALID_HANDLE_VALUE then begin
        pe32.dwSize := sizeof(TProcessEntry32);
        j := 0;
        setlength(ProcessList, j + 1);
        if Process32First(hProcSnapShot, pe32) then begin
          // first process
          ProcessList[j] := pe32.th32ProcessID;
          // walk the processes
          while Process32Next(hProcSnapShot, pe32) do begin
            Inc(j);
            setlength(ProcessList, j + 1);
            ProcessList[j] := pe32.th32ProcessID;
          end;
        end else begin // Process32First = False
          dwReturn := GetLastError;
        end;
        CloseHandle(hProcSnapShot);
      end else begin // hSnapShot = INVALID_HANDLE_VALUE
        dwReturn := GetLastError;
      end;
    end;
  end;
  result := dwReturn;
end;


function getProcessId(const AExeName: string): DWORD;
// return PID if process is running or 0 if not
var
  h: THandle;
  p: TProcessEntry32;
begin
  Result := 0;
  p.dwSize := SizeOf(p);
  h := CreateToolHelp32Snapshot(TH32CS_SnapProcess, 0);
  try
    Process32First(h, p);
    repeat
      if AnsiUpperCase(AExeName) = AnsiUpperCase(p.szExeFile) then begin
        Result := p.th32ProcessID;
      end;
    until (Result > 0) or (not Process32Next(h, p));
  finally
    CloseHandle(h);
  end;
end;

{ -----------------------------------------------------------------------------
  FUNKTION processExists(String)
  Checks if a process is running
}
function processExists(exeFileName: string): Boolean;
var
  ContinueLoop: BOOL;
  FSnapshotHandle: THandle;
  FProcessEntry32: TProcessEntry32;
begin
  FSnapshotHandle := CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
  FProcessEntry32.dwSize := SizeOf(FProcessEntry32);
  ContinueLoop := Process32First(FSnapshotHandle, FProcessEntry32);
  Result := False;
  while Integer(ContinueLoop) <> 0 do begin
    if ((UpperCase(ExtractFileName(FProcessEntry32.szExeFile)) =
      UpperCase(ExeFileName)) or (UpperCase(FProcessEntry32.szExeFile) =
      UpperCase(ExeFileName))) then begin
      //FProcessEntry32.th32ProcessID;
      Result := True;
    end;
    ContinueLoop := Process32Next(FSnapshotHandle, FProcessEntry32);
  end;
  CloseHandle(FSnapshotHandle);
end;
{ Example:
procedure TForm1.Button1Click(Sender: TObject);
begin
  if processExists('calc.exe') then
    ShowMessage('process is running')
  else
    ShowMessage('process NOT running');
end;
}

function MyEnumWindowProc(AHandle: THandle; LParam: LongWord): boolean;
stdcall;
var
  ProcessID: THandle;
begin
  ProcessID := 0;
  GetWindowThreadProcessID(AHandle, ProcessID);
  Result := not (ProcessID = LParam);
  if not Result then begin
    WindowHandle := AHandle;
  end;
end;

function GetWindowHandleByExeName(const AExeName: string): THandle;
var
  SnapShot: THandle;
  p: TProcessEntry32;
  ProcessHandle: THandle;
begin
  Result := 0;
  WindowHandle := 0;
  ProcessHandle := 0;
  p.dwSize := SizeOf(p);
  SnapShot := CreateToolhelp32Snapshot(TH32CS_SnapProcess, 0);
  try
    if Process32First(SnapShot, p) then begin
      repeat
        if AnsiLowerCase(AExeName) = AnsiLowerCase(p.szExeFile) then begin
          ProcessHandle := p.th32ProcessID;
        end;
      until (ProcessHandle <> 0) or not Process32Next(SnapShot, p);
    end;
    EnumWindows(@MyEnumWindowProc, ProcessHandle);
    Result := WindowHandle;
  finally
    CloseHandle(SnapShot);
  end;
end;


end.
